"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {
  CLEAR_RECORDS: true,
  FETCH_RECORDS: true,
  FETCH_RECORDS_SUCCESS: true,
  SAVE_RECORD: true,
  SAVE_RECORD_SUCCESS: true,
  DELETE_RECORD: true,
  DELETE_RECORD_FAIL: true,
  queryString: true,
  clearRecords: true,
  fetchRecords: true,
  fetchRecord: true,
  saveRecord: true,
  deleteRecord: true
};
exports["default"] = reducer;
exports.clearRecords = clearRecords;
exports.fetchRecords = fetchRecords;
exports.fetchRecord = fetchRecord;
exports.saveRecord = saveRecord;
exports.deleteRecord = deleteRecord;
exports.queryString = exports.DELETE_RECORD_FAIL = exports.DELETE_RECORD = exports.SAVE_RECORD_SUCCESS = exports.SAVE_RECORD = exports.FETCH_RECORDS_SUCCESS = exports.FETCH_RECORDS = exports.CLEAR_RECORDS = void 0;

var _qs = _interopRequireDefault(require("qs"));

var _jsonApiNormalizer = _interopRequireDefault(require("json-api-normalizer"));

var _deepmerge = _interopRequireDefault(require("deepmerge"));

var _selectors = require("./selectors");

Object.keys(_selectors).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function get() {
      return _selectors[key];
    }
  });
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var CLEAR_RECORDS = 'redux-json-api-module/api/CLEAR_RECORDS';
exports.CLEAR_RECORDS = CLEAR_RECORDS;
var FETCH_RECORDS = 'redux-json-api-module/api/FETCH_RECORDS';
exports.FETCH_RECORDS = FETCH_RECORDS;
var FETCH_RECORDS_SUCCESS = 'redux-json-api-module/api/FETCH_RECORDS_SUCCESS';
exports.FETCH_RECORDS_SUCCESS = FETCH_RECORDS_SUCCESS;
var SAVE_RECORD = 'redux-json-api-module/api/SAVE_RECORD';
exports.SAVE_RECORD = SAVE_RECORD;
var SAVE_RECORD_SUCCESS = 'redux-json-api-module/api/SAVE_RECORD_SUCCESS';
exports.SAVE_RECORD_SUCCESS = SAVE_RECORD_SUCCESS;
var DELETE_RECORD = 'redux-json-api-module/api/DELETE_RECORD';
exports.DELETE_RECORD = DELETE_RECORD;
var DELETE_RECORD_FAIL = 'redux-json-api-module/api/DELETE_RECORD_FAIL';
exports.DELETE_RECORD_FAIL = DELETE_RECORD_FAIL;
var INITIAL_STATE = {
  loading: false
}; // overwrite arrays
//   this could be improved by comparing array contents,
//   but arrays of objects will need to be converted to
//   JSON first which may be too expensive

var arrayMerge = function arrayMerge(a, b) {
  return b;
};

var mergeResult = function mergeResult(state, resp) {
  if (!resp.data) return state;
  return (0, _deepmerge["default"])(state, (0, _jsonApiNormalizer["default"])(resp, {
    camelizeKeys: false,
    camelizeTypeValues: false
  }), {
    arrayMerge: arrayMerge
  });
};

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : INITIAL_STATE;
  var action = arguments.length > 1 ? arguments[1] : undefined;

  switch (action.type) {
    case CLEAR_RECORDS:
      return _objectSpread({}, state, _defineProperty({}, action.record_type, {}));

    case FETCH_RECORDS_SUCCESS:
      return mergeResult(state, action.payload.data);

    case SAVE_RECORD_SUCCESS:
      return mergeResult(state, action.payload.data);

    case DELETE_RECORD:
      var recs = _objectSpread({}, state[action.record.type]);

      delete recs[action.record.id];
      return _objectSpread({}, state, _defineProperty({}, action.record.type, recs));

    case DELETE_RECORD_FAIL:
      return mergeResult(state, {
        data: action.meta.previousAction.record
      });

    default:
      return state;
  }
}

var queryString = function queryString(params) {
  return _qs["default"].stringify(params, {
    arrayFormat: 'brackets'
  });
};

exports.queryString = queryString;

function clearRecords(type) {
  return {
    type: CLEAR_RECORDS,
    record_type: type
  };
}

function fetchRecords(type) {
  var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return {
    type: FETCH_RECORDS,
    payload: {
      request: {
        method: 'GET',
        url: "/".concat(type, "?").concat(queryString(params))
      }
    }
  };
}

function fetchRecord(type, id) {
  var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return {
    type: FETCH_RECORDS,
    payload: {
      request: {
        method: 'GET',
        url: "/".concat(type, "/").concat(id, "?").concat(queryString(params))
      }
    }
  };
}

function saveRecord(record) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  console.info('saving record', record, options);
  var method = record.id ? 'PATCH' : 'POST';
  var url = "/".concat(record.type);
  if (record.id) url += "/".concat(record.id);
  return {
    type: SAVE_RECORD,
    ignoreFail: options.ignoreFail,
    payload: {
      request: {
        method: method,
        url: "".concat(url, "?").concat(queryString(options.params || {})),
        data: {
          data: record
        }
      }
    }
  };
}

function deleteRecord(record) {
  return {
    type: DELETE_RECORD,
    record: record,
    payload: {
      request: {
        method: 'DELETE',
        url: "".concat(record.type, "/").concat(record.id)
      }
    }
  };
}